from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="FormField")


@_attrs_define
class FormField:
    """
    Attributes:
        duid (str):
        form_duid (str):
        property_duid (str):
        locked (bool):
        order (str):
        required (bool):
        hidden (bool):
        label (str):
        default (Any):
        updated_by_client_duid (Union[None, Unset, str]):
    """

    duid: str
    form_duid: str
    property_duid: str
    locked: bool
    order: str
    required: bool
    hidden: bool
    label: str
    default: Any
    updated_by_client_duid: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        form_duid = self.form_duid

        property_duid = self.property_duid

        locked = self.locked

        order = self.order

        required = self.required

        hidden = self.hidden

        label = self.label

        default = self.default

        updated_by_client_duid: Union[None, Unset, str]
        if isinstance(self.updated_by_client_duid, Unset):
            updated_by_client_duid = UNSET
        else:
            updated_by_client_duid = self.updated_by_client_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "formDuid": form_duid,
                "propertyDuid": property_duid,
                "locked": locked,
                "order": order,
                "required": required,
                "hidden": hidden,
                "label": label,
                "default": default,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        form_duid = d.pop("formDuid")

        property_duid = d.pop("propertyDuid")

        locked = d.pop("locked")

        order = d.pop("order")

        required = d.pop("required")

        hidden = d.pop("hidden")

        label = d.pop("label")

        default = d.pop("default")

        def _parse_updated_by_client_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        updated_by_client_duid = _parse_updated_by_client_duid(d.pop("updatedByClientDuid", UNSET))

        form_field = cls(
            duid=duid,
            form_duid=form_duid,
            property_duid=property_duid,
            locked=locked,
            order=order,
            required=required,
            hidden=hidden,
            label=label,
            default=default,
            updated_by_client_duid=updated_by_client_duid,
        )

        form_field.additional_properties = d
        return form_field

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.icon_kind import IconKind
from ..models.view_kind import ViewKind
from ..types import UNSET, Unset

T = TypeVar("T", bound="View")


@_attrs_define
class View:
    """
    Attributes:
        duid (str):
        kind (ViewKind): * `Custom` - CUSTOM
            * `Search` - SEARCH
            * `Trash` - TRASH
            * `My tasks` - MY_TASKS
        accessible_by_team (bool):
        accessible_by_user_duids (List[str]):
        public (bool):
        order (str):
        title (str):
        description (str):
        icon_kind (IconKind): * `None` - NONE
            * `Icon` - ICON
            * `Emoji` - EMOJI
        icon_name_or_emoji (str):
        color_hex (str):
        layout_duid (str):
        favorited_by_user_duids (List[str]):
        always_shown_property_duids (List[str]):
        always_hidden_property_duids (List[str]):
        property_order_duids (List[str]):
        property_width_map (Any):
        updated_by_client_duid (Union[None, Unset, str]):
    """

    duid: str
    kind: ViewKind
    accessible_by_team: bool
    accessible_by_user_duids: List[str]
    public: bool
    order: str
    title: str
    description: str
    icon_kind: IconKind
    icon_name_or_emoji: str
    color_hex: str
    layout_duid: str
    favorited_by_user_duids: List[str]
    always_shown_property_duids: List[str]
    always_hidden_property_duids: List[str]
    property_order_duids: List[str]
    property_width_map: Any
    updated_by_client_duid: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        kind = self.kind.value

        accessible_by_team = self.accessible_by_team

        accessible_by_user_duids = self.accessible_by_user_duids

        public = self.public

        order = self.order

        title = self.title

        description = self.description

        icon_kind = self.icon_kind.value

        icon_name_or_emoji = self.icon_name_or_emoji

        color_hex = self.color_hex

        layout_duid = self.layout_duid

        favorited_by_user_duids = self.favorited_by_user_duids

        always_shown_property_duids = self.always_shown_property_duids

        always_hidden_property_duids = self.always_hidden_property_duids

        property_order_duids = self.property_order_duids

        property_width_map = self.property_width_map

        updated_by_client_duid: Union[None, Unset, str]
        if isinstance(self.updated_by_client_duid, Unset):
            updated_by_client_duid = UNSET
        else:
            updated_by_client_duid = self.updated_by_client_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "kind": kind,
                "accessibleByTeam": accessible_by_team,
                "accessibleByUserDuids": accessible_by_user_duids,
                "public": public,
                "order": order,
                "title": title,
                "description": description,
                "iconKind": icon_kind,
                "iconNameOrEmoji": icon_name_or_emoji,
                "colorHex": color_hex,
                "layoutDuid": layout_duid,
                "favoritedByUserDuids": favorited_by_user_duids,
                "alwaysShownPropertyDuids": always_shown_property_duids,
                "alwaysHiddenPropertyDuids": always_hidden_property_duids,
                "propertyOrderDuids": property_order_duids,
                "propertyWidthMap": property_width_map,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        kind = ViewKind(d.pop("kind"))

        accessible_by_team = d.pop("accessibleByTeam")

        accessible_by_user_duids = cast(List[str], d.pop("accessibleByUserDuids"))

        public = d.pop("public")

        order = d.pop("order")

        title = d.pop("title")

        description = d.pop("description")

        icon_kind = IconKind(d.pop("iconKind"))

        icon_name_or_emoji = d.pop("iconNameOrEmoji")

        color_hex = d.pop("colorHex")

        layout_duid = d.pop("layoutDuid")

        favorited_by_user_duids = cast(List[str], d.pop("favoritedByUserDuids"))

        always_shown_property_duids = cast(List[str], d.pop("alwaysShownPropertyDuids"))

        always_hidden_property_duids = cast(List[str], d.pop("alwaysHiddenPropertyDuids"))

        property_order_duids = cast(List[str], d.pop("propertyOrderDuids"))

        property_width_map = d.pop("propertyWidthMap")

        def _parse_updated_by_client_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        updated_by_client_duid = _parse_updated_by_client_duid(d.pop("updatedByClientDuid", UNSET))

        view = cls(
            duid=duid,
            kind=kind,
            accessible_by_team=accessible_by_team,
            accessible_by_user_duids=accessible_by_user_duids,
            public=public,
            order=order,
            title=title,
            description=description,
            icon_kind=icon_kind,
            icon_name_or_emoji=icon_name_or_emoji,
            color_hex=color_hex,
            layout_duid=layout_duid,
            favorited_by_user_duids=favorited_by_user_duids,
            always_shown_property_duids=always_shown_property_duids,
            always_hidden_property_duids=always_hidden_property_duids,
            property_order_duids=property_order_duids,
            property_width_map=property_width_map,
            updated_by_client_duid=updated_by_client_duid,
        )

        view.additional_properties = d
        return view

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Required for type hinting compatibility when using Python 3.9
from __future__ import annotations
from .generated.models import *
from .dart import (
    Dart,
    begin_task,
    cli,
    create_task,
    get_dartboards,
    get_folders,
    get_host,
    is_logged_in,
    login,
    replicate_space,
    set_host,
    update_dartboard,
    update_folder,
    update_task,
)
from .webhook import is_signature_correct



from enum import Enum


class UserDataEntityRetrieveEntityKind(str, Enum):
    ATTACHMENTS = "attachments"
    COMMENTS = "comments"
    DARTBOARDS = "dartboards"
    DASHBOARDS = "dashboards"
    DOCS = "docs"
    FOLDERS = "folders"
    FORMS = "forms"
    FORM_FIELDS = "form-fields"
    LAYOUTS = "layouts"
    LINKS = "links"
    OPTIONS = "options"
    PROPERTIES = "properties"
    REACTIONS = "reactions"
    RELATIONSHIPS = "relationships"
    RELATIONSHIP_KINDS = "relationship-kinds"
    SPACES = "spaces"
    STATUSES = "statuses"
    TASKS = "tasks"
    TASK_DOC_RELATIONSHIPS = "task-doc-relationships"
    TASK_KINDS = "task-kinds"
    TENANTS = "tenants"
    USERS = "users"
    USER_DARTBOARD_LAYOUTS = "user-dartboard-layouts"
    VIEWS = "views"
    WEBHOOKS = "webhooks"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.dartboard_kind import DartboardKind
from ..models.icon_kind import IconKind
from ..types import UNSET, Unset

T = TypeVar("T", bound="DartboardCreate")


@_attrs_define
class DartboardCreate:
    """
    Attributes:
        duid (str):
        space_duid (str):
        order (str):
        kind (Union[Unset, DartboardKind]): * `Active` - ACTIVE
            * `Next` - NEXT
            * `Backlog` - BACKLOG
            * `Finished` - FINISHED
            * `Custom` - CUSTOM
        title (Union[Unset, str]):
        description (Union[Unset, str]):
        icon_kind (Union[Unset, IconKind]): * `None` - NONE
            * `Icon` - ICON
            * `Emoji` - EMOJI
        icon_name_or_emoji (Union[Unset, str]):
        color_hex (Union[Unset, str]):
        index (Union[None, Unset, int]):
        started_at (Union[None, Unset, datetime.datetime]):
        finished_at (Union[None, Unset, datetime.datetime]):
        default_property_map (Union[Unset, Any]):
        always_shown_property_duids (Union[Unset, List[str]]):
        always_hidden_property_duids (Union[Unset, List[str]]):
        property_order_duids (Union[Unset, List[str]]):
        property_width_map (Union[Unset, Any]):
    """

    duid: str
    space_duid: str
    order: str
    kind: Union[Unset, DartboardKind] = UNSET
    title: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    icon_kind: Union[Unset, IconKind] = UNSET
    icon_name_or_emoji: Union[Unset, str] = UNSET
    color_hex: Union[Unset, str] = UNSET
    index: Union[None, Unset, int] = UNSET
    started_at: Union[None, Unset, datetime.datetime] = UNSET
    finished_at: Union[None, Unset, datetime.datetime] = UNSET
    default_property_map: Union[Unset, Any] = UNSET
    always_shown_property_duids: Union[Unset, List[str]] = UNSET
    always_hidden_property_duids: Union[Unset, List[str]] = UNSET
    property_order_duids: Union[Unset, List[str]] = UNSET
    property_width_map: Union[Unset, Any] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        space_duid = self.space_duid

        order = self.order

        kind: Union[Unset, str] = UNSET
        if not isinstance(self.kind, Unset):
            kind = self.kind.value

        title = self.title

        description = self.description

        icon_kind: Union[Unset, str] = UNSET
        if not isinstance(self.icon_kind, Unset):
            icon_kind = self.icon_kind.value

        icon_name_or_emoji = self.icon_name_or_emoji

        color_hex = self.color_hex

        index: Union[None, Unset, int]
        if isinstance(self.index, Unset):
            index = UNSET
        else:
            index = self.index

        started_at: Union[None, Unset, str]
        if isinstance(self.started_at, Unset):
            started_at = UNSET
        elif isinstance(self.started_at, datetime.datetime):
            started_at = self.started_at.isoformat()
        else:
            started_at = self.started_at

        finished_at: Union[None, Unset, str]
        if isinstance(self.finished_at, Unset):
            finished_at = UNSET
        elif isinstance(self.finished_at, datetime.datetime):
            finished_at = self.finished_at.isoformat()
        else:
            finished_at = self.finished_at

        default_property_map = self.default_property_map

        always_shown_property_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.always_shown_property_duids, Unset):
            always_shown_property_duids = self.always_shown_property_duids

        always_hidden_property_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.always_hidden_property_duids, Unset):
            always_hidden_property_duids = self.always_hidden_property_duids

        property_order_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.property_order_duids, Unset):
            property_order_duids = self.property_order_duids

        property_width_map = self.property_width_map

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "spaceDuid": space_duid,
                "order": order,
            }
        )
        if kind is not UNSET:
            field_dict["kind"] = kind
        if title is not UNSET:
            field_dict["title"] = title
        if description is not UNSET:
            field_dict["description"] = description
        if icon_kind is not UNSET:
            field_dict["iconKind"] = icon_kind
        if icon_name_or_emoji is not UNSET:
            field_dict["iconNameOrEmoji"] = icon_name_or_emoji
        if color_hex is not UNSET:
            field_dict["colorHex"] = color_hex
        if index is not UNSET:
            field_dict["index"] = index
        if started_at is not UNSET:
            field_dict["startedAt"] = started_at
        if finished_at is not UNSET:
            field_dict["finishedAt"] = finished_at
        if default_property_map is not UNSET:
            field_dict["defaultPropertyMap"] = default_property_map
        if always_shown_property_duids is not UNSET:
            field_dict["alwaysShownPropertyDuids"] = always_shown_property_duids
        if always_hidden_property_duids is not UNSET:
            field_dict["alwaysHiddenPropertyDuids"] = always_hidden_property_duids
        if property_order_duids is not UNSET:
            field_dict["propertyOrderDuids"] = property_order_duids
        if property_width_map is not UNSET:
            field_dict["propertyWidthMap"] = property_width_map

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        space_duid = d.pop("spaceDuid")

        order = d.pop("order")

        _kind = d.pop("kind", UNSET)
        kind: Union[Unset, DartboardKind]
        if isinstance(_kind, Unset):
            kind = UNSET
        else:
            kind = DartboardKind(_kind)

        title = d.pop("title", UNSET)

        description = d.pop("description", UNSET)

        _icon_kind = d.pop("iconKind", UNSET)
        icon_kind: Union[Unset, IconKind]
        if isinstance(_icon_kind, Unset):
            icon_kind = UNSET
        else:
            icon_kind = IconKind(_icon_kind)

        icon_name_or_emoji = d.pop("iconNameOrEmoji", UNSET)

        color_hex = d.pop("colorHex", UNSET)

        def _parse_index(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        index = _parse_index(d.pop("index", UNSET))

        def _parse_started_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                started_at_type_0 = isoparse(data)

                return started_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        started_at = _parse_started_at(d.pop("startedAt", UNSET))

        def _parse_finished_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                finished_at_type_0 = isoparse(data)

                return finished_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        finished_at = _parse_finished_at(d.pop("finishedAt", UNSET))

        default_property_map = d.pop("defaultPropertyMap", UNSET)

        always_shown_property_duids = cast(List[str], d.pop("alwaysShownPropertyDuids", UNSET))

        always_hidden_property_duids = cast(List[str], d.pop("alwaysHiddenPropertyDuids", UNSET))

        property_order_duids = cast(List[str], d.pop("propertyOrderDuids", UNSET))

        property_width_map = d.pop("propertyWidthMap", UNSET)

        dartboard_create = cls(
            duid=duid,
            space_duid=space_duid,
            order=order,
            kind=kind,
            title=title,
            description=description,
            icon_kind=icon_kind,
            icon_name_or_emoji=icon_name_or_emoji,
            color_hex=color_hex,
            index=index,
            started_at=started_at,
            finished_at=finished_at,
            default_property_map=default_property_map,
            always_shown_property_duids=always_shown_property_duids,
            always_hidden_property_duids=always_hidden_property_duids,
            property_order_duids=property_order_duids,
            property_width_map=property_width_map,
        )

        dartboard_create.additional_properties = d
        return dartboard_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.attachment import Attachment
from ...models.comment import Comment
from ...models.comment_reaction import CommentReaction
from ...models.dartboard import Dartboard
from ...models.dashboard import Dashboard
from ...models.doc import Doc
from ...models.folder import Folder
from ...models.form import Form
from ...models.form_field import FormField
from ...models.layout import Layout
from ...models.option import Option
from ...models.property_ import Property
from ...models.relationship import Relationship
from ...models.relationship_kind import RelationshipKind
from ...models.space import Space
from ...models.status import Status
from ...models.task import Task
from ...models.task_doc_relationship import TaskDocRelationship
from ...models.task_kind import TaskKind
from ...models.task_link import TaskLink
from ...models.tenant import Tenant
from ...models.user import User
from ...models.user_dartboard_layout import UserDartboardLayout
from ...models.user_data_entity_retrieve_entity_kind import UserDataEntityRetrieveEntityKind
from ...models.view import View
from ...models.webhook import Webhook
from ...types import Response


def _get_kwargs(
    entity_kind: UserDataEntityRetrieveEntityKind,
    entity_duid: str,
) -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": f"/api/v0/{entity_kind}/{entity_duid}",
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[
    Union[
        "Attachment",
        "Comment",
        "CommentReaction",
        "Dartboard",
        "Dashboard",
        "Doc",
        "Folder",
        "Form",
        "FormField",
        "Layout",
        "Option",
        "Property",
        "Relationship",
        "RelationshipKind",
        "Space",
        "Status",
        "Task",
        "TaskDocRelationship",
        "TaskKind",
        "TaskLink",
        "Tenant",
        "User",
        "UserDartboardLayout",
        "View",
        "Webhook",
    ]
]:
    if response.status_code == HTTPStatus.OK:

        def _parse_response_200(
            data: object,
        ) -> Union[
            "Attachment",
            "Comment",
            "CommentReaction",
            "Dartboard",
            "Dashboard",
            "Doc",
            "Folder",
            "Form",
            "FormField",
            "Layout",
            "Option",
            "Property",
            "Relationship",
            "RelationshipKind",
            "Space",
            "Status",
            "Task",
            "TaskDocRelationship",
            "TaskKind",
            "TaskLink",
            "Tenant",
            "User",
            "UserDartboardLayout",
            "View",
            "Webhook",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_0 = Attachment.from_dict(data)

                return componentsschemas_user_data_entity_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_1 = Comment.from_dict(data)

                return componentsschemas_user_data_entity_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_2 = CommentReaction.from_dict(data)

                return componentsschemas_user_data_entity_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_3 = User.from_dict(data)

                return componentsschemas_user_data_entity_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_4 = Dartboard.from_dict(data)

                return componentsschemas_user_data_entity_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_5 = Dashboard.from_dict(data)

                return componentsschemas_user_data_entity_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_6 = TaskKind.from_dict(data)

                return componentsschemas_user_data_entity_type_6
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_7 = Doc.from_dict(data)

                return componentsschemas_user_data_entity_type_7
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_8 = Folder.from_dict(data)

                return componentsschemas_user_data_entity_type_8
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_9 = Form.from_dict(data)

                return componentsschemas_user_data_entity_type_9
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_10 = FormField.from_dict(data)

                return componentsschemas_user_data_entity_type_10
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_11 = Layout.from_dict(data)

                return componentsschemas_user_data_entity_type_11
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_12 = Option.from_dict(data)

                return componentsschemas_user_data_entity_type_12
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_13 = Property.from_dict(data)

                return componentsschemas_user_data_entity_type_13
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_14 = Relationship.from_dict(data)

                return componentsschemas_user_data_entity_type_14
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_15 = RelationshipKind.from_dict(data)

                return componentsschemas_user_data_entity_type_15
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_16 = Space.from_dict(data)

                return componentsschemas_user_data_entity_type_16
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_17 = Status.from_dict(data)

                return componentsschemas_user_data_entity_type_17
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_18 = Task.from_dict(data)

                return componentsschemas_user_data_entity_type_18
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_19 = TaskDocRelationship.from_dict(data)

                return componentsschemas_user_data_entity_type_19
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_20 = TaskLink.from_dict(data)

                return componentsschemas_user_data_entity_type_20
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_21 = Tenant.from_dict(data)

                return componentsschemas_user_data_entity_type_21
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_22 = UserDartboardLayout.from_dict(data)

                return componentsschemas_user_data_entity_type_22
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_user_data_entity_type_23 = View.from_dict(data)

                return componentsschemas_user_data_entity_type_23
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_user_data_entity_type_24 = Webhook.from_dict(data)

            return componentsschemas_user_data_entity_type_24

        response_200 = _parse_response_200(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[
    Union[
        "Attachment",
        "Comment",
        "CommentReaction",
        "Dartboard",
        "Dashboard",
        "Doc",
        "Folder",
        "Form",
        "FormField",
        "Layout",
        "Option",
        "Property",
        "Relationship",
        "RelationshipKind",
        "Space",
        "Status",
        "Task",
        "TaskDocRelationship",
        "TaskKind",
        "TaskLink",
        "Tenant",
        "User",
        "UserDartboardLayout",
        "View",
        "Webhook",
    ]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    entity_kind: UserDataEntityRetrieveEntityKind,
    entity_duid: str,
    *,
    client: AuthenticatedClient,
) -> Response[
    Union[
        "Attachment",
        "Comment",
        "CommentReaction",
        "Dartboard",
        "Dashboard",
        "Doc",
        "Folder",
        "Form",
        "FormField",
        "Layout",
        "Option",
        "Property",
        "Relationship",
        "RelationshipKind",
        "Space",
        "Status",
        "Task",
        "TaskDocRelationship",
        "TaskKind",
        "TaskLink",
        "Tenant",
        "User",
        "UserDartboardLayout",
        "View",
        "Webhook",
    ]
]:
    """
    Args:
        entity_kind (UserDataEntityRetrieveEntityKind):
        entity_duid (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union['Attachment', 'Comment', 'CommentReaction', 'Dartboard', 'Dashboard', 'Doc', 'Folder', 'Form', 'FormField', 'Layout', 'Option', 'Property', 'Relationship', 'RelationshipKind', 'Space', 'Status', 'Task', 'TaskDocRelationship', 'TaskKind', 'TaskLink', 'Tenant', 'User', 'UserDartboardLayout', 'View', 'Webhook']]
    """

    kwargs = _get_kwargs(
        entity_kind=entity_kind,
        entity_duid=entity_duid,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    entity_kind: UserDataEntityRetrieveEntityKind,
    entity_duid: str,
    *,
    client: AuthenticatedClient,
) -> Optional[
    Union[
        "Attachment",
        "Comment",
        "CommentReaction",
        "Dartboard",
        "Dashboard",
        "Doc",
        "Folder",
        "Form",
        "FormField",
        "Layout",
        "Option",
        "Property",
        "Relationship",
        "RelationshipKind",
        "Space",
        "Status",
        "Task",
        "TaskDocRelationship",
        "TaskKind",
        "TaskLink",
        "Tenant",
        "User",
        "UserDartboardLayout",
        "View",
        "Webhook",
    ]
]:
    """
    Args:
        entity_kind (UserDataEntityRetrieveEntityKind):
        entity_duid (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union['Attachment', 'Comment', 'CommentReaction', 'Dartboard', 'Dashboard', 'Doc', 'Folder', 'Form', 'FormField', 'Layout', 'Option', 'Property', 'Relationship', 'RelationshipKind', 'Space', 'Status', 'Task', 'TaskDocRelationship', 'TaskKind', 'TaskLink', 'Tenant', 'User', 'UserDartboardLayout', 'View', 'Webhook']
    """

    return sync_detailed(
        entity_kind=entity_kind,
        entity_duid=entity_duid,
        client=client,
    ).parsed


async def asyncio_detailed(
    entity_kind: UserDataEntityRetrieveEntityKind,
    entity_duid: str,
    *,
    client: AuthenticatedClient,
) -> Response[
    Union[
        "Attachment",
        "Comment",
        "CommentReaction",
        "Dartboard",
        "Dashboard",
        "Doc",
        "Folder",
        "Form",
        "FormField",
        "Layout",
        "Option",
        "Property",
        "Relationship",
        "RelationshipKind",
        "Space",
        "Status",
        "Task",
        "TaskDocRelationship",
        "TaskKind",
        "TaskLink",
        "Tenant",
        "User",
        "UserDartboardLayout",
        "View",
        "Webhook",
    ]
]:
    """
    Args:
        entity_kind (UserDataEntityRetrieveEntityKind):
        entity_duid (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union['Attachment', 'Comment', 'CommentReaction', 'Dartboard', 'Dashboard', 'Doc', 'Folder', 'Form', 'FormField', 'Layout', 'Option', 'Property', 'Relationship', 'RelationshipKind', 'Space', 'Status', 'Task', 'TaskDocRelationship', 'TaskKind', 'TaskLink', 'Tenant', 'User', 'UserDartboardLayout', 'View', 'Webhook']]
    """

    kwargs = _get_kwargs(
        entity_kind=entity_kind,
        entity_duid=entity_duid,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    entity_kind: UserDataEntityRetrieveEntityKind,
    entity_duid: str,
    *,
    client: AuthenticatedClient,
) -> Optional[
    Union[
        "Attachment",
        "Comment",
        "CommentReaction",
        "Dartboard",
        "Dashboard",
        "Doc",
        "Folder",
        "Form",
        "FormField",
        "Layout",
        "Option",
        "Property",
        "Relationship",
        "RelationshipKind",
        "Space",
        "Status",
        "Task",
        "TaskDocRelationship",
        "TaskKind",
        "TaskLink",
        "Tenant",
        "User",
        "UserDartboardLayout",
        "View",
        "Webhook",
    ]
]:
    """
    Args:
        entity_kind (UserDataEntityRetrieveEntityKind):
        entity_duid (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union['Attachment', 'Comment', 'CommentReaction', 'Dartboard', 'Dashboard', 'Doc', 'Folder', 'Form', 'FormField', 'Layout', 'Option', 'Property', 'Relationship', 'RelationshipKind', 'Space', 'Status', 'Task', 'TaskDocRelationship', 'TaskKind', 'TaskLink', 'Tenant', 'User', 'UserDartboardLayout', 'View', 'Webhook']
    """

    return (
        await asyncio_detailed(
            entity_kind=entity_kind,
            entity_duid=entity_duid,
            client=client,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="GoogleData")


@_attrs_define
class GoogleData:
    """
    Attributes:
        name (str):
        email (str):
        picture_url (Union[None, str]):
        organization_domain (Union[None, str]):
    """

    name: str
    email: str
    picture_url: Union[None, str]
    organization_domain: Union[None, str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        email = self.email

        picture_url: Union[None, str]
        picture_url = self.picture_url

        organization_domain: Union[None, str]
        organization_domain = self.organization_domain

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "email": email,
                "pictureUrl": picture_url,
                "organizationDomain": organization_domain,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        email = d.pop("email")

        def _parse_picture_url(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        picture_url = _parse_picture_url(d.pop("pictureUrl"))

        def _parse_organization_domain(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        organization_domain = _parse_organization_domain(d.pop("organizationDomain"))

        google_data = cls(
            name=name,
            email=email,
            picture_url=picture_url,
            organization_domain=organization_domain,
        )

        google_data.additional_properties = d
        return google_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.subscription import Subscription

if TYPE_CHECKING:
    from ..models.discord_integration import DiscordIntegration
    from ..models.github_integration import GithubIntegration
    from ..models.notion_integration import NotionIntegration
    from ..models.saml_config import SamlConfig
    from ..models.slack_integration import SlackIntegration
    from ..models.zapier_integration import ZapierIntegration


T = TypeVar("T", bound="Tenant")


@_attrs_define
class Tenant:
    """
    Attributes:
        duid (str):
        is_dart (bool):
        name (str):
        created_at (datetime.datetime):
        timezone (str):
        subscription (Subscription): * `Personal` - PERSONAL
            * `Premium` - PREMIUM
        entitlement_overrides (Any):
        image_url (Union[None, str]):
        backlog_enabled (bool):
        ai_assignment_enabled (bool):
        email_integration_enabled (bool):
        close_parent_on_close_all_subtasks (bool):
        move_subtasks_on_move_parent (bool):
        update_subtasks_status_on_update_parent_status (bool):
        copy_parent_fields_on_create (bool):
        update_blockee_dates_on_update_blocker_due_date (bool):
        webhook_enabled (bool):
        webhook_secret (str):
        saml_config (Union['SamlConfig', None]):
        notion_integration (Union['NotionIntegration', None]):
        slack_integration (Union['SlackIntegration', None]):
        discord_integration (Union['DiscordIntegration', None]):
        github_integration (Union['GithubIntegration', None]):
        zapier_integration (Union['ZapierIntegration', None]):
    """

    duid: str
    is_dart: bool
    name: str
    created_at: datetime.datetime
    timezone: str
    subscription: Subscription
    entitlement_overrides: Any
    image_url: Union[None, str]
    backlog_enabled: bool
    ai_assignment_enabled: bool
    email_integration_enabled: bool
    close_parent_on_close_all_subtasks: bool
    move_subtasks_on_move_parent: bool
    update_subtasks_status_on_update_parent_status: bool
    copy_parent_fields_on_create: bool
    update_blockee_dates_on_update_blocker_due_date: bool
    webhook_enabled: bool
    webhook_secret: str
    saml_config: Union["SamlConfig", None]
    notion_integration: Union["NotionIntegration", None]
    slack_integration: Union["SlackIntegration", None]
    discord_integration: Union["DiscordIntegration", None]
    github_integration: Union["GithubIntegration", None]
    zapier_integration: Union["ZapierIntegration", None]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.discord_integration import DiscordIntegration
        from ..models.github_integration import GithubIntegration
        from ..models.notion_integration import NotionIntegration
        from ..models.saml_config import SamlConfig
        from ..models.slack_integration import SlackIntegration
        from ..models.zapier_integration import ZapierIntegration

        duid = self.duid

        is_dart = self.is_dart

        name = self.name

        created_at = self.created_at.isoformat()

        timezone = self.timezone

        subscription = self.subscription.value

        entitlement_overrides = self.entitlement_overrides

        image_url: Union[None, str]
        image_url = self.image_url

        backlog_enabled = self.backlog_enabled

        ai_assignment_enabled = self.ai_assignment_enabled

        email_integration_enabled = self.email_integration_enabled

        close_parent_on_close_all_subtasks = self.close_parent_on_close_all_subtasks

        move_subtasks_on_move_parent = self.move_subtasks_on_move_parent

        update_subtasks_status_on_update_parent_status = self.update_subtasks_status_on_update_parent_status

        copy_parent_fields_on_create = self.copy_parent_fields_on_create

        update_blockee_dates_on_update_blocker_due_date = self.update_blockee_dates_on_update_blocker_due_date

        webhook_enabled = self.webhook_enabled

        webhook_secret = self.webhook_secret

        saml_config: Union[Dict[str, Any], None]
        if isinstance(self.saml_config, SamlConfig):
            saml_config = self.saml_config.to_dict()
        else:
            saml_config = self.saml_config

        notion_integration: Union[Dict[str, Any], None]
        if isinstance(self.notion_integration, NotionIntegration):
            notion_integration = self.notion_integration.to_dict()
        else:
            notion_integration = self.notion_integration

        slack_integration: Union[Dict[str, Any], None]
        if isinstance(self.slack_integration, SlackIntegration):
            slack_integration = self.slack_integration.to_dict()
        else:
            slack_integration = self.slack_integration

        discord_integration: Union[Dict[str, Any], None]
        if isinstance(self.discord_integration, DiscordIntegration):
            discord_integration = self.discord_integration.to_dict()
        else:
            discord_integration = self.discord_integration

        github_integration: Union[Dict[str, Any], None]
        if isinstance(self.github_integration, GithubIntegration):
            github_integration = self.github_integration.to_dict()
        else:
            github_integration = self.github_integration

        zapier_integration: Union[Dict[str, Any], None]
        if isinstance(self.zapier_integration, ZapierIntegration):
            zapier_integration = self.zapier_integration.to_dict()
        else:
            zapier_integration = self.zapier_integration

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "isDart": is_dart,
                "name": name,
                "createdAt": created_at,
                "timezone": timezone,
                "subscription": subscription,
                "entitlementOverrides": entitlement_overrides,
                "imageUrl": image_url,
                "backlogEnabled": backlog_enabled,
                "aiAssignmentEnabled": ai_assignment_enabled,
                "emailIntegrationEnabled": email_integration_enabled,
                "closeParentOnCloseAllSubtasks": close_parent_on_close_all_subtasks,
                "moveSubtasksOnMoveParent": move_subtasks_on_move_parent,
                "updateSubtasksStatusOnUpdateParentStatus": update_subtasks_status_on_update_parent_status,
                "copyParentFieldsOnCreate": copy_parent_fields_on_create,
                "updateBlockeeDatesOnUpdateBlockerDueDate": update_blockee_dates_on_update_blocker_due_date,
                "webhookEnabled": webhook_enabled,
                "webhookSecret": webhook_secret,
                "samlConfig": saml_config,
                "notionIntegration": notion_integration,
                "slackIntegration": slack_integration,
                "discordIntegration": discord_integration,
                "githubIntegration": github_integration,
                "zapierIntegration": zapier_integration,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.discord_integration import DiscordIntegration
        from ..models.github_integration import GithubIntegration
        from ..models.notion_integration import NotionIntegration
        from ..models.saml_config import SamlConfig
        from ..models.slack_integration import SlackIntegration
        from ..models.zapier_integration import ZapierIntegration

        d = src_dict.copy()
        duid = d.pop("duid")

        is_dart = d.pop("isDart")

        name = d.pop("name")

        created_at = isoparse(d.pop("createdAt"))

        timezone = d.pop("timezone")

        subscription = Subscription(d.pop("subscription"))

        entitlement_overrides = d.pop("entitlementOverrides")

        def _parse_image_url(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        image_url = _parse_image_url(d.pop("imageUrl"))

        backlog_enabled = d.pop("backlogEnabled")

        ai_assignment_enabled = d.pop("aiAssignmentEnabled")

        email_integration_enabled = d.pop("emailIntegrationEnabled")

        close_parent_on_close_all_subtasks = d.pop("closeParentOnCloseAllSubtasks")

        move_subtasks_on_move_parent = d.pop("moveSubtasksOnMoveParent")

        update_subtasks_status_on_update_parent_status = d.pop("updateSubtasksStatusOnUpdateParentStatus")

        copy_parent_fields_on_create = d.pop("copyParentFieldsOnCreate")

        update_blockee_dates_on_update_blocker_due_date = d.pop("updateBlockeeDatesOnUpdateBlockerDueDate")

        webhook_enabled = d.pop("webhookEnabled")

        webhook_secret = d.pop("webhookSecret")

        def _parse_saml_config(data: object) -> Union["SamlConfig", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                saml_config_type_0 = SamlConfig.from_dict(data)

                return saml_config_type_0
            except:  # noqa: E722
                pass
            return cast(Union["SamlConfig", None], data)

        saml_config = _parse_saml_config(d.pop("samlConfig"))

        def _parse_notion_integration(data: object) -> Union["NotionIntegration", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                notion_integration_type_0 = NotionIntegration.from_dict(data)

                return notion_integration_type_0
            except:  # noqa: E722
                pass
            return cast(Union["NotionIntegration", None], data)

        notion_integration = _parse_notion_integration(d.pop("notionIntegration"))

        def _parse_slack_integration(data: object) -> Union["SlackIntegration", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                slack_integration_type_0 = SlackIntegration.from_dict(data)

                return slack_integration_type_0
            except:  # noqa: E722
                pass
            return cast(Union["SlackIntegration", None], data)

        slack_integration = _parse_slack_integration(d.pop("slackIntegration"))

        def _parse_discord_integration(data: object) -> Union["DiscordIntegration", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                discord_integration_type_0 = DiscordIntegration.from_dict(data)

                return discord_integration_type_0
            except:  # noqa: E722
                pass
            return cast(Union["DiscordIntegration", None], data)

        discord_integration = _parse_discord_integration(d.pop("discordIntegration"))

        def _parse_github_integration(data: object) -> Union["GithubIntegration", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                github_integration_type_0 = GithubIntegration.from_dict(data)

                return github_integration_type_0
            except:  # noqa: E722
                pass
            return cast(Union["GithubIntegration", None], data)

        github_integration = _parse_github_integration(d.pop("githubIntegration"))

        def _parse_zapier_integration(data: object) -> Union["ZapierIntegration", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                zapier_integration_type_0 = ZapierIntegration.from_dict(data)

                return zapier_integration_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ZapierIntegration", None], data)

        zapier_integration = _parse_zapier_integration(d.pop("zapierIntegration"))

        tenant = cls(
            duid=duid,
            is_dart=is_dart,
            name=name,
            created_at=created_at,
            timezone=timezone,
            subscription=subscription,
            entitlement_overrides=entitlement_overrides,
            image_url=image_url,
            backlog_enabled=backlog_enabled,
            ai_assignment_enabled=ai_assignment_enabled,
            email_integration_enabled=email_integration_enabled,
            close_parent_on_close_all_subtasks=close_parent_on_close_all_subtasks,
            move_subtasks_on_move_parent=move_subtasks_on_move_parent,
            update_subtasks_status_on_update_parent_status=update_subtasks_status_on_update_parent_status,
            copy_parent_fields_on_create=copy_parent_fields_on_create,
            update_blockee_dates_on_update_blocker_due_date=update_blockee_dates_on_update_blocker_due_date,
            webhook_enabled=webhook_enabled,
            webhook_secret=webhook_secret,
            saml_config=saml_config,
            notion_integration=notion_integration,
            slack_integration=slack_integration,
            discord_integration=discord_integration,
            github_integration=github_integration,
            zapier_integration=zapier_integration,
        )

        tenant.additional_properties = d
        return tenant

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="BurnUpChartAdtl")


@_attrs_define
class BurnUpChartAdtl:
    """
    Attributes:
        start_date (Union[None, datetime.date]):
        end_date (Union[None, datetime.date]):
    """

    start_date: Union[None, datetime.date]
    end_date: Union[None, datetime.date]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        start_date: Union[None, str]
        if isinstance(self.start_date, datetime.date):
            start_date = self.start_date.isoformat()
        else:
            start_date = self.start_date

        end_date: Union[None, str]
        if isinstance(self.end_date, datetime.date):
            end_date = self.end_date.isoformat()
        else:
            end_date = self.end_date

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "startDate": start_date,
                "endDate": end_date,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_start_date(data: object) -> Union[None, datetime.date]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                start_date_type_0 = isoparse(data).date()

                return start_date_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.date], data)

        start_date = _parse_start_date(d.pop("startDate"))

        def _parse_end_date(data: object) -> Union[None, datetime.date]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                end_date_type_0 = isoparse(data).date()

                return end_date_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.date], data)

        end_date = _parse_end_date(d.pop("endDate"))

        burn_up_chart_adtl = cls(
            start_date=start_date,
            end_date=end_date,
        )

        burn_up_chart_adtl.additional_properties = d
        return burn_up_chart_adtl

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.chart_type import ChartType

if TYPE_CHECKING:
    from ..models.bar_chart_adtl import BarChartAdtl
    from ..models.burn_up_chart_adtl import BurnUpChartAdtl
    from ..models.line_chart_adtl import LineChartAdtl
    from ..models.number_chart_adtl import NumberChartAdtl
    from ..models.pie_chart_adtl import PieChartAdtl
    from ..models.table_chart_adtl import TableChartAdtl


T = TypeVar("T", bound="Chart")


@_attrs_define
class Chart:
    """
    Attributes:
        duid (str):
        type (ChartType): * `bar` - BAR
            * `burn-up` - BURN_UP
            * `line` - LINE
            * `number` - NUMBER
            * `pie` - PIE
            * `table` - TABLE
            * `text` - TEXT_BLOCK
        title (str):
        x (int):
        y (int):
        width (int):
        height (int):
        adtl (Union['BarChartAdtl', 'BurnUpChartAdtl', 'LineChartAdtl', 'NumberChartAdtl', 'PieChartAdtl',
            'TableChartAdtl']):
    """

    duid: str
    type: ChartType
    title: str
    x: int
    y: int
    width: int
    height: int
    adtl: Union["BarChartAdtl", "BurnUpChartAdtl", "LineChartAdtl", "NumberChartAdtl", "PieChartAdtl", "TableChartAdtl"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.bar_chart_adtl import BarChartAdtl
        from ..models.burn_up_chart_adtl import BurnUpChartAdtl
        from ..models.line_chart_adtl import LineChartAdtl
        from ..models.number_chart_adtl import NumberChartAdtl
        from ..models.pie_chart_adtl import PieChartAdtl

        duid = self.duid

        type = self.type.value

        title = self.title

        x = self.x

        y = self.y

        width = self.width

        height = self.height

        adtl: Dict[str, Any]
        if isinstance(self.adtl, BarChartAdtl):
            adtl = self.adtl.to_dict()
        elif isinstance(self.adtl, BurnUpChartAdtl):
            adtl = self.adtl.to_dict()
        elif isinstance(self.adtl, LineChartAdtl):
            adtl = self.adtl.to_dict()
        elif isinstance(self.adtl, NumberChartAdtl):
            adtl = self.adtl.to_dict()
        elif isinstance(self.adtl, PieChartAdtl):
            adtl = self.adtl.to_dict()
        else:
            adtl = self.adtl.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "type": type,
                "title": title,
                "x": x,
                "y": y,
                "width": width,
                "height": height,
                "adtl": adtl,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.bar_chart_adtl import BarChartAdtl
        from ..models.burn_up_chart_adtl import BurnUpChartAdtl
        from ..models.line_chart_adtl import LineChartAdtl
        from ..models.number_chart_adtl import NumberChartAdtl
        from ..models.pie_chart_adtl import PieChartAdtl
        from ..models.table_chart_adtl import TableChartAdtl

        d = src_dict.copy()
        duid = d.pop("duid")

        type = ChartType(d.pop("type"))

        title = d.pop("title")

        x = d.pop("x")

        y = d.pop("y")

        width = d.pop("width")

        height = d.pop("height")

        def _parse_adtl(
            data: object,
        ) -> Union[
            "BarChartAdtl", "BurnUpChartAdtl", "LineChartAdtl", "NumberChartAdtl", "PieChartAdtl", "TableChartAdtl"
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_chart_adtl_type_0 = BarChartAdtl.from_dict(data)

                return componentsschemas_chart_adtl_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_chart_adtl_type_1 = BurnUpChartAdtl.from_dict(data)

                return componentsschemas_chart_adtl_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_chart_adtl_type_2 = LineChartAdtl.from_dict(data)

                return componentsschemas_chart_adtl_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_chart_adtl_type_3 = NumberChartAdtl.from_dict(data)

                return componentsschemas_chart_adtl_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_chart_adtl_type_4 = PieChartAdtl.from_dict(data)

                return componentsschemas_chart_adtl_type_4
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_chart_adtl_type_5 = TableChartAdtl.from_dict(data)

            return componentsschemas_chart_adtl_type_5

        adtl = _parse_adtl(d.pop("adtl"))

        chart = cls(
            duid=duid,
            type=type,
            title=title,
            x=x,
            y=y,
            width=width,
            height=height,
            adtl=adtl,
        )

        chart.additional_properties = d
        return chart

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ChartType(str, Enum):
    BAR = "bar"
    BURN_UP = "burn-up"
    LINE = "line"
    NUMBER = "number"
    PIE = "pie"
    TABLE = "table"
    TEXT = "text"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="WebhookUpdate")


@_attrs_define
class WebhookUpdate:
    """
    Attributes:
        duid (str):
        order (Union[Unset, str]):
        url (Union[Unset, str]):
    """

    duid: str
    order: Union[Unset, str] = UNSET
    url: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        order = self.order

        url = self.url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
            }
        )
        if order is not UNSET:
            field_dict["order"] = order
        if url is not UNSET:
            field_dict["url"] = url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        order = d.pop("order", UNSET)

        url = d.pop("url", UNSET)

        webhook_update = cls(
            duid=duid,
            order=order,
            url=url,
        )

        webhook_update.additional_properties = d
        return webhook_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.event import Event


T = TypeVar("T", bound="Notification")


@_attrs_define
class Notification:
    """
    Attributes:
        duid (str):
        created_at (datetime.datetime):
        user_duid (str):
        event (Event):
        read (bool):
        updated_by_client_duid (Union[None, Unset, str]):
    """

    duid: str
    created_at: datetime.datetime
    user_duid: str
    event: "Event"
    read: bool
    updated_by_client_duid: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        created_at = self.created_at.isoformat()

        user_duid = self.user_duid

        event = self.event.to_dict()

        read = self.read

        updated_by_client_duid: Union[None, Unset, str]
        if isinstance(self.updated_by_client_duid, Unset):
            updated_by_client_duid = UNSET
        else:
            updated_by_client_duid = self.updated_by_client_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "createdAt": created_at,
                "userDuid": user_duid,
                "event": event,
                "read": read,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.event import Event

        d = src_dict.copy()
        duid = d.pop("duid")

        created_at = isoparse(d.pop("createdAt"))

        user_duid = d.pop("userDuid")

        event = Event.from_dict(d.pop("event"))

        read = d.pop("read")

        def _parse_updated_by_client_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        updated_by_client_duid = _parse_updated_by_client_duid(d.pop("updatedByClientDuid", UNSET))

        notification = cls(
            duid=duid,
            created_at=created_at,
            user_duid=user_duid,
            event=event,
            read=read,
            updated_by_client_duid=updated_by_client_duid,
        )

        notification.additional_properties = d
        return notification

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class UserRole(str, Enum):
    ADMIN = "Admin"
    FINANCIAL_ADMIN = "Financial admin"
    GUEST = "Guest"
    MEMBER = "Member"
    TECHNICAL_ADMIN = "Technical admin"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class DocsListOItem(str, Enum):
    RECENT = "recent"
    TITLE = "title"
    VALUE_0 = "-recent"
    VALUE_1 = "-title"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.theme import Theme
from ..models.user_role import UserRole
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserUpdate")


@_attrs_define
class UserUpdate:
    """
    Attributes:
        duid (str):
        role (Union[Unset, UserRole]): * `Admin` - ADMIN
            * `Member` - MEMBER
            * `Guest` - GUEST
            * `Technical admin` - TECHNICAL_ADMIN
            * `Financial admin` - FINANCIAL_ADMIN
        name (Union[Unset, str]):
        theme (Union[Unset, Theme]): * `System Default` - SYSTEM_DEFAULT
            * `Light` - LIGHT
            * `Dark` - DARK
        color_hex (Union[Unset, str]):
        open_in_native_app (Union[Unset, bool]):
        last_url_path (Union[None, Unset, str]):
        first_day_of_week (Union[Unset, int]):
        sections (Union[Unset, Any]):
        layout (Union[Unset, Any]):
        notification_default (Union[Unset, bool]):
        notification_in_app (Union[Unset, bool]):
        notification_email (Union[Unset, bool]):
        notification_slack (Union[Unset, bool]):
    """

    duid: str
    role: Union[Unset, UserRole] = UNSET
    name: Union[Unset, str] = UNSET
    theme: Union[Unset, Theme] = UNSET
    color_hex: Union[Unset, str] = UNSET
    open_in_native_app: Union[Unset, bool] = UNSET
    last_url_path: Union[None, Unset, str] = UNSET
    first_day_of_week: Union[Unset, int] = UNSET
    sections: Union[Unset, Any] = UNSET
    layout: Union[Unset, Any] = UNSET
    notification_default: Union[Unset, bool] = UNSET
    notification_in_app: Union[Unset, bool] = UNSET
    notification_email: Union[Unset, bool] = UNSET
    notification_slack: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        name = self.name

        theme: Union[Unset, str] = UNSET
        if not isinstance(self.theme, Unset):
            theme = self.theme.value

        color_hex = self.color_hex

        open_in_native_app = self.open_in_native_app

        last_url_path: Union[None, Unset, str]
        if isinstance(self.last_url_path, Unset):
            last_url_path = UNSET
        else:
            last_url_path = self.last_url_path

        first_day_of_week = self.first_day_of_week

        sections = self.sections

        layout = self.layout

        notification_default = self.notification_default

        notification_in_app = self.notification_in_app

        notification_email = self.notification_email

        notification_slack = self.notification_slack

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
            }
        )
        if role is not UNSET:
            field_dict["role"] = role
        if name is not UNSET:
            field_dict["name"] = name
        if theme is not UNSET:
            field_dict["theme"] = theme
        if color_hex is not UNSET:
            field_dict["colorHex"] = color_hex
        if open_in_native_app is not UNSET:
            field_dict["openInNativeApp"] = open_in_native_app
        if last_url_path is not UNSET:
            field_dict["lastUrlPath"] = last_url_path
        if first_day_of_week is not UNSET:
            field_dict["firstDayOfWeek"] = first_day_of_week
        if sections is not UNSET:
            field_dict["sections"] = sections
        if layout is not UNSET:
            field_dict["layout"] = layout
        if notification_default is not UNSET:
            field_dict["notificationDefault"] = notification_default
        if notification_in_app is not UNSET:
            field_dict["notificationInApp"] = notification_in_app
        if notification_email is not UNSET:
            field_dict["notificationEmail"] = notification_email
        if notification_slack is not UNSET:
            field_dict["notificationSlack"] = notification_slack

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        _role = d.pop("role", UNSET)
        role: Union[Unset, UserRole]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = UserRole(_role)

        name = d.pop("name", UNSET)

        _theme = d.pop("theme", UNSET)
        theme: Union[Unset, Theme]
        if isinstance(_theme, Unset):
            theme = UNSET
        else:
            theme = Theme(_theme)

        color_hex = d.pop("colorHex", UNSET)

        open_in_native_app = d.pop("openInNativeApp", UNSET)

        def _parse_last_url_path(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        last_url_path = _parse_last_url_path(d.pop("lastUrlPath", UNSET))

        first_day_of_week = d.pop("firstDayOfWeek", UNSET)

        sections = d.pop("sections", UNSET)

        layout = d.pop("layout", UNSET)

        notification_default = d.pop("notificationDefault", UNSET)

        notification_in_app = d.pop("notificationInApp", UNSET)

        notification_email = d.pop("notificationEmail", UNSET)

        notification_slack = d.pop("notificationSlack", UNSET)

        user_update = cls(
            duid=duid,
            role=role,
            name=name,
            theme=theme,
            color_hex=color_hex,
            open_in_native_app=open_in_native_app,
            last_url_path=last_url_path,
            first_day_of_week=first_day_of_week,
            sections=sections,
            layout=layout,
            notification_default=notification_default,
            notification_in_app=notification_in_app,
            notification_email=notification_email,
            notification_slack=notification_slack,
        )

        user_update.additional_properties = d
        return user_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
